
ccc   Subroutine on "convert_string_date.f":
ccc    string2date      - extracts a date from a string which may
ccc                       include additional leading information
ccc                       (but no trailing text)
ccc    date2string      - appends a date to a string, after any
ccc                       pre-existing text
ccc    swap_date        - primarily used for swapping
ccc                       from dd-mm-yyyy to yyyy-mm-dd
ccc
ccc   Note: These subroutines contain Y10K bugs.
ccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



c     ********************
c     General description:
c     ********************

c     The subroutine "string2date" extracts a date from a given string.
c     The string may contain leading information which is placed in
c     front of the date, with a blank character separation
c      (e.g., "days since 2000-10-22 14:30:00.0"  or
c             "2000-10-22-14:30:00.0")
c     No information can be included after the date.


c     ***********************
c     Input/output variables:
c     ***********************

c      string         (s)  - input string which contains a date,
c                             format: 'text Y-M-D h:m:s' or
c                                     'Y-M-D h:m:s'
c                              where
c                               Y is a (multidigit)       year'
c                               M is a (one or two digit) month
c                               D is a (one or two digit) day
c                               h is a (one or two digit) hour
c                               m is a (one or two digit) minute
c                               s is   (real value)       seconds
c      date           (iA) - output date, format:
c                             date(1) - year
c                             date(2) - month
c                             date(3) - day
c                             date(4) - hour
c                             date(5) - minute
c      seconds        (r)  - output, seconds to add to date

c     Above:
c      s  - string           [ character*(*) ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      r  - real             [ 4 byte, aka. "float" ]



      subroutine string2date(string,date,seconds)

      implicit none

c     Input:
      character*(*) string

c     Output:
      integer date(5)
      real seconds

c     Local:
      integer position,length,new_position

c     Functions:
      integer str_len,str_pos


c     Initialize:
      position = str_len(string)

c     Find seconds:
      new_position = str_pos(string,-position,':')
      if (new_position .le. 1) go to 999
      length = position - new_position
      if (length .lt. 1) go to 999
      read(string(new_position+1:position),*)seconds
      position = new_position - 1

c     Find minute:
      new_position = str_pos(string,-position,':')
      if (new_position .le. 1) go to 999
      length = position - new_position
      if (length .lt. 1 .or. length .gt. 2) go to 999
      read(string(new_position+1:position),*)date(5)
      position = new_position - 1

c     Find hour:
      new_position = max(str_pos(string,-position,' '),
     >                   str_pos(string,-position,'-'))
      if (new_position .le. 1) go to 999
      length = position - new_position
      if (length .lt. 1 .or. length .gt. 2) go to 999
      read(string(new_position+1:position),*)date(4)
      position = new_position - 1

c     Find day:
      new_position = str_pos(string,-position,'-')
      if (new_position .le. 1) go to 999
      length = position - new_position
      if (length .lt. 1 .or. length .gt. 2) go to 999
      read(string(new_position+1:position),*)date(3)
      position = new_position - 1

c     Find month:
      new_position = str_pos(string,-position,'-')
      if (new_position .le. 1) go to 999
      length = position - new_position
      if (length .lt. 1 .or. length .gt. 2) go to 999
      read(string(new_position+1:position),*)date(2)
      position = new_position - 1

c     Find year:
      new_position = str_pos(string,-position,' ')
      length = position - new_position
      if (length .lt. 1) go to 999
      read(string(new_position+1:position),*)date(1)


      go to 1000

 999  write(*,*)
      write(*,*)'Error in the subroutine "string2date".'
      write(*,*)' Unrecognized format.'
      write(*,*)'  The input string'
      write(*,*)'   ',string
      write(*,*)'  should end with:   Y-M-D h:m:s or'
      write(*,*)'                     Y-M-D-h:m:s'
      write(*,*)'  where Y is a (multidigit)       year'
      write(*,*)'        M is a (one or two digit) month'
      write(*,*)'        D is a (one or two digit) day'
      write(*,*)'        h is a (one or two digit) hour'
      write(*,*)'        m is a (one or two digit) minute'
      write(*,*)'        s is   (real value)       seconds'
      write(*,*)'  Note:'
      write(*,*)"   1. The '-', ':' and ' ' or '-' in ",
     >          "front of 'h' must all be present"
      write(*,*)'   2. If the string contains additional leading ',
     >          'information,'
      write(*,*)'      this information must be separated from the date'
      write(*,*)'      by one or more blank characters'
      write(*,*)'Terminates.'
      write(*,*)
      stop

 1000 return
      end



c     ********************
c     General description:
c     ********************

c     The subroutine "date2string" appends a date to a string,
c     provided that the string's length allows such an action.
c     The string may contain leading information which is placed in
c     front of the date, a blank character separation will be used
c     between the leading information and the date.
c      (e.g., ""           -> "2000-10-22 14:30:00.0"  or
c             "days since" -> "days since 2000-10-22-14:30:00.0")


c     ***********************
c     Input/output variables:
c     ***********************

c      size           (i)  - input: the limit of no. of characters in
c                                   'string' (according to the declaration)
c                            output: the actual size of 'string' after
c                                    the date has been included
c      decimals       (i)  - the no. of decimals for seconds (if zero,
c                             the decimal point will be omitted in the
c                             output string)
c      date           (iA) - input date, format:
c                             date(1) - year
c                             date(2) - month
c                             date(3) - day
c                             date(4) - hour
c                             date(5) - minute
c      seconds        (r)  - input, seconds to add to 'date'
c      string         (s)  - input string to which a date may be added
c                             (may be empty)
c                             format on exit: 'text Y-M-D h:m:s' or
c                                             'Y-M-D h:m:s'
c                              where
c                               Y is a (multidigit) year'
c                               M is a (two digit)  month
c                               D is a (two digit)  day
c                               h is a (two digit)  hour
c                               m is a (two digit)  minute
c                               s is   (real value) seconds

c     Above:
c      i  - integer          [ 4 byte, aka. "long" ]
c      s  - string           [ character*(*) ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      r  - real             [ 4 byte, aka. "float" ]


      subroutine date2string(size,decimals,date,seconds,string)

      implicit none

c     Input:
      integer decimals,date(5)
      real seconds

c     Input/output:
      integer size

c     Output:
      character*(*) string

c     Local:
      integer org_size,length,number,fraction,secs
      character*22 fmt_txt22
      character*25 fmt_txt25
      character*30 fmt_txt30

c     Functions:
      integer str_len


c     Set present length of string, disregarding trailing blank
c     characters, add trailing blank if non-blank characters exist:
      org_size = str_len(string)
      if (org_size .gt. 0) then
         org_size = org_size + 1
         string(org_size:org_size) = ' '
      end if

c     Set length of output string:
      if (decimals .le. 0) then
         length = org_size + 19
      else
         length = org_size + 20 + decimals
      end if

c     Check size:
      if (length .gt. size) go to 999

      if (decimals .le. 0) then

         if (org_size .gt. 0) then
            write(fmt_txt22,101)org_size
 101        format('(a',i3.3,',i4.4,5(a1,i2.2))')
            write(string,fmt_txt22)string(1:org_size),
     >                             date(1),'-',date(2),'-',date(3),' ',
     >                             date(4),':',date(5),':',nint(seconds)
         else
 102        format(i4.4,5(a1,i2.2))
            write(string,102)date(1),'-',date(2),'-',date(3),' ',
     >                       date(4),':',date(5),':',nint(seconds)
         end if

      else

         number = nint( (10.**decimals)*seconds )
         secs = number/(10**decimals)
         fraction = mod(number, 10**decimals)

         if (org_size .gt. 0) then
            write(fmt_txt30,103)org_size,decimals,decimals
 103        format('(a',i3.3,',i4.4,5(a1,i2.2),a1,i',i1,'.',i1,')')
            write(string,fmt_txt30)string(1:org_size),
     >                             date(1),'-',date(2),'-',date(3),' ',
     >                             date(4),':',date(5),':',
     >                             secs,'.',fraction
         else
            write(fmt_txt25,104)decimals,decimals
 104        format('(i4.4,5(a1,i2.2),a1,i',i1,'.',i1,')')
            write(string,fmt_txt25)date(1),'-',date(2),'-',date(3),' ',
     >                             date(4),':',date(5),':',
     >                             secs,'.',fraction
         end if

      end if

      size = length

      go to 1000

 999  write(*,*)
      write(*,*)'Error in the subroutine "date2string".'
      write(*,*)' Too short string!'
      write(*,*)'  Size according to input specification:',size
      write(*,*)'  Required size to store date information:',length
      write(*,*)'Terminates.'
      write(*,*)
      stop

 1000 return
      end



c     ********************
c     General description:
c     ********************

c     If the date (string) format is unknown, or of a type that
c     contains a month name (in letters), and/or reversed format
c     (dd-mm-yyyy, not yyyy-mm-dd), the date string will be converted
c     to the recognized format (yyyy-mm-dd) by the subroutine
c     "swap_date". Note that the original input string is generally
c     modified on output.


c     ***********************
c     Input/output variables:
c     ***********************

c      string         (s)  - input/output string that contains a date

c     Above:
c      s  - string           [ character*(*) ]


      subroutine swap_date(string)

      implicit none

c     Input/output:
      character*(*) string

c     Local:
      integer size,a,z,zero,nine,n,indx,last,dd,mm,yyyy,dig_1,dig_pos
      character*2   mnth
      character*10  date
      character*120 lc_string,tail

      character*3 month(12)
      data month / 'jan', 'feb', 'mar', 'apr', 'may', 'jun',
     >             'jul', 'aug', 'sep', 'oct', 'nov', 'dec' /

c     Functions:
      integer str_len
      character null_chrctr
      character*120 lowercase


      size = str_len(string)
      if (size .gt. 120) go to 999
      lc_string = lowercase(string)

c     Store ASCII codes:
      a     = ichar('a')
      z     = ichar('Z')
      zero  = ichar('0')
      nine  = ichar('9')


c     Replace character month?

      indx = 0
      n = 0
      do while (indx .eq. 0 .and. n .lt. 12)
         n = n + 1
         indx = index(lc_string,month(n))
      end do

      if (indx .gt. 0) then

c        Replace!

c        Find index of last character:
         last = indx + 1
         do while (ichar(lc_string(last:last)) .ge. a .and.
     >             ichar(lc_string(last:last)) .ge. z .and.
     >             last .lt. size)
            last = last + 1
         end do

c        Convert month no. to character*2:
         write(mnth,101)n
 101     format(i2.2)

c        Rewrite original string:
         if (last .lt. size) then
            tail = mnth
            n = str_len(mnth)
            tail = tail(1:n) // string(last:size)
            n = str_len(tail)
            string = string(1:indx-1) // tail(1:n)
            n = str_len(string)
            size = n
         else
            string = string(1:indx-1) // mnth
            n = indx + 1
         end if

c        Fill org. string with blanks:
         do while (n .lt. size)
            n = n + 1
            string(n:n)= ' '
         end do

      end if


c     Assume that the year is 4 digits.

c     Convert from DD-MM-YYYY to YYYY-MM-DD?

c     Require that the format is 'ddddsddsdd' or 'ddsddsdddd'
c      ('d' is a digit, 's' is a separator) 

c     Look for first digit:
      n = 1
      do while ( (ichar(string(n:n)) .lt. zero .or.
     >            ichar(string(n:n)) .gt. nine)
     >           .and. n .lt. size )
         n = n + 1
      end do
      if (n .eq. size) write(*,*)n,size
      if (n .eq. size) go to 998
      dig_1 = n

c     Find position of next separator:
      do while ( (ichar(string(n:n)) .ge. zero .and.
     >            ichar(string(n:n)) .le. nine) .and.
     >          n .lt. size )
         write(*,*)
         n = n + 1
      end do

      if (n-dig_1 .eq. 4) then

         write(*,*)'Assuming date format yyyy-mm-dd in:',string(1:size)

      else

c        Store day:
         if (n-dig_1 .gt. 2) go to 998
         read(string(dig_1:n-1),*)dd

c        Find next digit:
         n = n + 1
         do while ( (ichar(string(n:n)) .lt. zero .or.
     >               ichar(string(n:n)) .gt. nine)
     >             .and. n .lt. size )
            n = n + 1
         end do
         if (n .eq. size) go to 998
         dig_pos = n

c        Find next separator:
         do while ( (ichar(string(n:n)) .ge. zero .and.
     >               ichar(string(n:n)) .le. nine)
     >             .and. n .lt. size )
            n = n + 1
         end do

c        Store month:
         if (n-dig_pos .gt. 2) go to 998
         read(string(dig_pos:n-1),*)mm
         if (mm .gt. 12) go to 998

c        Find next digit:
         n = n + 1
         do while ( (ichar(string(n:n)) .lt. zero .or.
     >               ichar(string(n:n)) .gt. nine)
     >             .and. n .lt. size )
            n = n + 1
         end do
         if (n .eq. size) go to 998
         if (size-n .lt. 3) go to 998
         dig_pos = n

c        Find next non-numeral:
         do while ( (ichar(string(n:n)) .ge. zero .and.
     >               ichar(string(n:n)) .le. nine)
     >             .and. n .lt. size )
            n = n + 1
         end do
         if (n .eq. size .and.
     >       ichar(string(n:n)) .ge. zero .and.
     >       ichar(string(n:n)) .le. nine) n = n + 1

c        Store year:
         if (n-dig_pos .ne. 4) go to 997
         read(string(dig_pos:n-1),*)yyyy

c        Backup of tail of 'string':
         tail = ''
         if (n .le. size) tail = string(n:size)

c        Write warning when assuming DD-MM-YYYY, as opposed to MM-DD-YYYY:
         write(*,*)' WARNING! Found a substring that will be assumed to'
         write(*,*)'          have the format dd-mm-yyyy: ',
     >             string(dig_1:n-1)

c        Reorganise 'string':
         write(date,111)yyyy,'-',mm,'-',dd
 111     format(i4.4,a1,i2.2,a1,i2.2)
         string = ''
         if (dig_1 .gt. 1) then
            string = lc_string(1:dig_1-1) // date
         else
            string = date
         end if
         if (n .le. size) string = string(1:dig_1+9) // tail

      end if

      go to 1000

 997  write(*,*)
      write(*,*)'Warning from "swap_date":'
      write(*,*)' Could not find 4 digit year in: ',string(1:size)
      write(*,*)' (above, the string may have been modified)'
      write(*,*)' Assuming y-m-d, continuing ...'
      write(*,*)
      go to 1000

 998  write(*,*)
      write(*,*)'Error in the subroutine "swap_date".'
      write(*,*)' The string does not contain a recognized date.'
      write(*,*)'  (recognized date formats: yyyy-mm-dd or dd-mm-yyyy)'
      write(*,*)' Input string (possibly modified): ',string(1:size)
      write(*,*)'Terminates.'
      write(*,*)
      stop

 999  write(*,*)
      write(*,*)'Error in the subroutine "swap_date".'
      write(*,*)' Too long string!'
      write(*,*)'  Size of input string:',size
      write(*,*)'  Max size:            ',120
      write(*,*)'Terminates.'
      write(*,*)
      stop

 1000 return
      end

