
c     Subroutine on this file:
c      write_all_nc_met_headers   - main subroutine for creating
c                                    netCDF headers

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c     - NOTES FOR "write_all_nc_met_headers" -

c     **************
c     General notes:
c     **************

c     1. The path of the include file ('netdcf.inc') must be
c        included when compiling!; e.g.,
c          f77 -c -I/usr/local/netcdf-3.4/include write_all_nc_headers.f

c     2. "write_all_nc_met_headers" requires that the file which will
c        contain the header (see the file "specs_met.inc") has not been
c        opened outside "write_all_nc_met_headers".

c     3. "write_all_nc_met_headers" does not close the netCDF-file,
c        but exits in "data mode".


c     *********************************************
c     List of implemented parameters and variables:
c     *********************************************

c      (*1), (*2), (*3) : See comments below

c      parameter (short/long) type  dimensions  accuracy units      id
c      ^^^^^^^^^^^^^^^^^^^^^^ ^^^^  ^^^^^^^^^^  ^^^^^^^^ ^^^^^      ^^

c     COMPULSORY:
c      lon / Longitude        float (*1)        N/A      degrees    lon_id
c      lat / Latitude         float (*1)        N/A      degrees    lat_id
c      time / Time            float nt          N/A      (*1)       time_id

c     ... COMPULSORY IF VERTICAL RESOLUTION:
c      level / (*2)           int4  nz          (*3)     (*3)       level_id

c     ANY OF:
c      temp / Temperature     int2  nx,ny,nz,nt 0.01     deg.
c                                                        Kelvin     temp_id
c      rough / Surface
c              roughness      int2  nx,ny,nt    0.000001 m          rough_id
c      cldcvr / Mean cloud
c               cover         int2  nx,ny,nt    0.001    none       cldcvr_id
c      u / Wind-x             int2  nx,ny,nz,nt 0.01     m/s        u_id
c      v / Wind-y             int2  nx,ny,nz,nt 0.01     m/s        v_id
c      topo / Topography      float nx,ny       N/A      m          topo_id


c     (*1):
c     'time' is set relative to 'reference_date', with units as specified
c     by 'timeunit'. These are both set in "specs.inc", and recognized
c     values of 'timeunit' are:
c      'seconds'
c      'minutes'
c      'hours'
c      'days'
c      'months'
c      'years'

c     (*2):
c     'lon'/'lat' dimensions depend on the grid ('sphere_on' in "specs.inc"):
c      parameter
c       (short/long name)   'sphere_on'  dimensions
c      ^^^^^^^^^^^^^^^^^^   ^^^^^^^^^^^  ^^^^^^^^^^
c      lon / Longitude      0            nx,ny
c      lon / Longitude      1            nx
c      lat / Latitude       0            nx,ny
c      lat / Latitude       1            ny

c     (*3):
c     The possible types of vertical levels ('leveltype' in "specs.inc") are:
c      type levels            accuracy  units
c      ^^^^ ^^^^^^            ^^^^^^^^  ^^^^^
c      1    Layers            N/A       none
c      2    z (surface = 0)   1.0       m
c      3    sigma (positive)  0.001     none
c      4    sigma (negative)  0.001     none

c     Misc.:
c      int2: 2 byte integer (aka. "short")
c      int4: 4 byte integer (aka. "long")


      subroutine write_all_nc_met_headers()


      implicit none

      include 'netcdf.inc'
      include 'id_met.inc'
      include 'specs_met.inc'

c     Local:
      integer dim_options
      parameter (dim_options = 8)
      integer nc_status,size,filesize,var_n_dim(dim_options),
     >        var_dim_ids(4,dim_options),option_no
      real range(2)
      logical exists
      character reply
      integer i

c     Function:
      integer str_len


      filesize = str_len(filename)


c     ************
c     netCDF Open:
c     ************

c     Open file, check existence:
      nc_status = NF_CREATE(filename, NF_NOCLOBBER, nc_id)
      if (nc_status .ne. NF_NOERR) then
         write(*,*)
         write(*,*)'The file you specified:'
         write(*,*)'    ',filename(1:filesize)
         write(*,*)' probably exists!'
         write(*,*)'Continue to add additional header information',
     >             ' to this file? (y/n)'
         read(*,'(a1)')reply
         do while (reply .ne. 'y' .and. reply .ne. 'n')
            write(*,*)'Please reply by typing one of',
     >                " the characters 'y' or 'n':"
            read(*,'(a1)')reply
         end do
         if (reply .eq. 'n') then
            write(*,*)
            write(*,*)'Terminates as requested.'
            write(*,*)
            stop
         else
c           Create file failed, retry opening file, put in define mode:
            nc_status = NF_OPEN(filename, NF_WRITE, nc_id)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 1)
            nc_status = NF_REDEF(nc_id)
            if (nc_status .ne. NF_NOERR) call handle_err(nc_status, 2)
         end if

         exists = .true.

      else

         exists = .false.

      end if

      write(*,*)
      write(*,*)'================================================='
      if (.not. exists) then
         write(*,*)' Attempting to create header for the netCDF file'
      else
         write(*,*)' Attempting to modify header for the netCDF file'
      end if
      write(*,*)'     ',filename(1:filesize)
      write(*,*)'================================================='
      write(*,*)



c     *************************
c     netCDF global attributes:
c     *************************

c     General information:
c     --------------------

      call nc_char_attrib(NF_GLOBAL,'title',title,'global')
      call nc_char_attrib(NF_GLOBAL,'source',source,'global')
      call nc_char_attrib(NF_GLOBAL,'method',method,'global')
      call nc_char_attrib(NF_GLOBAL,'reference',reference,'global')
      call nc_char_attrib(NF_GLOBAL,'web_reference',web_reference,
     >                    'global')
      call nc_char_attrib(NF_GLOBAL,'history',history,'global')
      call nc_char_attrib(NF_GLOBAL,'author',author,'global')
      call nc_char_attrib(NF_GLOBAL,'email',email,'global')

c     --- START DNMI conversions: ---
      call nc_float_attrib(NF_GLOBAL,'grid',6,grid,'grid')
c     --- END DNMI conversions: ---


c     ******************
c     netCDF dimensions:
c     ******************

      write(*,*)
      write(*,*)'Dimensions:'

      if (sphere_on .eq. 1) then
         call write_nc_dim_header('lon',nx,x_id)
         call write_nc_dim_header('lat',ny,y_id)
      else
         call write_nc_dim_header('x',nx,x_id)
         call write_nc_dim_header('y',ny,y_id)
      end if
      if (nz .gt. 0) call write_nc_dim_header('z',nz,z_id)
      call write_nc_dim_header('time',NF_UNLIMITED,t_id)
      if (climatology_on .eq. 1)
     >   call write_nc_dim_header('month',12,m_id)


c     --- START set arrays with dimension id-s ---

c     option 1: x (longitude)
      var_n_dim(1) = 1
      var_dim_ids(1,1) = x_id

c     option 2: y (latitude)
      var_n_dim(2) = 1
      var_dim_ids(1,2) = y_id

c     option 3: z
      var_n_dim(3) = 1
      var_dim_ids(1,3) = z_id

c     option 4: N/A
      var_n_dim(4) = 1
      var_dim_ids(1,4) = 0

c     option 5: t
      var_n_dim(5) = 1
      var_dim_ids(1,5) = t_id

c     option 6: x,y
      var_n_dim(6) = 2
      var_dim_ids(1,6) = x_id
      var_dim_ids(2,6) = y_id

c     option 7: x,y,t
      var_n_dim(7) = 3
      var_dim_ids(1,7) = x_id
      var_dim_ids(2,7) = y_id
      var_dim_ids(3,7) = t_id

c     option 8: x,y,z,t
      var_n_dim(8) = 4
      var_dim_ids(1,8) = x_id
      var_dim_ids(2,8) = y_id
      var_dim_ids(3,8) = z_id
      var_dim_ids(4,8) = t_id

c     --- END set arrays with dimension id-s ---


c     *****************
c        ***********

c     *****************
c     netCDF Variables:
c     *****************


c     Grid specification:
c     %%%%%%%%%%%%%%%%%%%


      write(*,*)
      write(*,*)'Grid specification:'


c     Longitude:
      call write_nc_var_header
     >        ('lon',NF_FLOAT,dim_options,var_n_dim,var_dim_ids,
     >         lon_id,1,0.,0.)
      range(1) = -360.
      range(2) = 720.
      call nc_float_attrib(lon_id,'valid_range',2,range,'lon')

c     Latitude:
      call write_nc_var_header
     >        ('lat',NF_FLOAT,dim_options,var_n_dim,var_dim_ids,
     >         lat_id,1,0.,0.)
      range(1) = -90.
      range(2) = 90.
      call nc_float_attrib(lat_id,'valid_range',2,range,'lat')

      if (nz .gt. 0)
c        Levels:
     >      call write_nc_var_header
     >              ('level',NF_INT,dim_options,var_n_dim,var_dim_ids,
     >               level_id,1,level_offset,level_scale)

c     Time:
      call write_nc_time_header(dim_options,var_n_dim,var_dim_ids)
      if (climatology_on .eq. 1)
c        Month:
     >   call add_nc_month_header()


c     Meteorology:
c     %%%%%%%%%%%%

      write(*,*)
      write(*,*)'Meteorology:'


      if (nz .gt. 0) then

c        Temperature:
         if (temp_on .eq. 1)
     >      call write_nc_var_header
     >              ('temp',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               temp_id,1,temp_offset,temp_scale)

c        Temperature (Kelvin):
         if (temp_K_on .eq. 1)
     >      call write_nc_var_header
     >              ('temp_K',NF_SHORT,
     >               dim_options,var_n_dim,var_dim_ids,
     >               temp_K_id,1,temp_K_offset,temp_K_scale)

c        Layer thickess:
         if (leveltype .eq. 1 .and. thck_on .eq. 1)
     >      call write_nc_var_header
     >              ('thck',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               thck_id,1,thck_offset,thck_scale)

      end if

c     Surface roughness:
      if (rough_on .eq. 1)
     >   call write_nc_var_header
     >           ('rough',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >            rough_id,1,rough_offset,rough_scale)

c     Mean cloud cover:
      if (cldcvr_on .eq. 1)
     >   call write_nc_var_header
     >           ('cldcvr',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >            cldcvr_id,1,cldcvr_offset,cldcvr_scale)


c     Kinematics:
c     %%%%%%%%%%%

      write(*,*)
      write(*,*)'Kinematics:'


      if (nz .gt. 0) then

c        Wind-x:
         if (u_on .eq. 1)
     >      call write_nc_var_header
     >              ('u',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               u_id,1,u_offset,u_scale)

c        Wind-y:
         if (v_on .eq. 1)
     >      call write_nc_var_header
     >              ('v',NF_SHORT,dim_options,var_n_dim,var_dim_ids,
     >               v_id,1,v_offset,v_scale)

      end if


c     Topography:
c     %%%%%%%%%%%

      write(*,*)
      write(*,*)'Topography:'


c     Bottom topography:

      if (topo_on .eq. 1)
     >   call write_nc_var_header
     >           ('topo',NF_FLOAT,dim_options,var_n_dim,var_dim_ids,
     >            topo_id,1,0.,1.)


c     ***********************
c          *************

c     ***********************
c     netCDF end definitions:
c     ***********************

      nc_status = NF_ENDDEF(nc_id)
      if (nc_status .ne. NF_NOERR) call handle_err(nc_status,53)

      write(*,*)
      write(*,*)'========================================='
      if (.not. exists) then
         write(*,*)' Done creating netCDF header on the file'
      else
         write(*,*)' Done modifying netCDF header on the file'
      end if
      write(*,*)'     ',filename(1:filesize)
      write(*,*)' The file has been set in "data mode".'
      write(*,*)'========================================='
      write(*,*)


      end
